<?php

namespace Schaffen\Gef\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Http\Utils\Response;
use Illuminate\Http\Request;

use Schaffen\Gef\Http\Requests;
use Prettus\Service\Contracts\ServiceInterface;
use Prettus\Service\Exceptions\ServiceException;
use Schaffen\Gef\Http\Requests\ApontamentoVeiculoGastoCreateRequest;
use Schaffen\Gef\Http\Requests\ApontamentoVeiculoGastoUpdateRequest;
use Schaffen\Gef\Models\ApontamentoVeiculoGasto;
use Schaffen\Gef\Models\ApontamentoVeiculoGastoMvf;
use Schaffen\Gef\Repositories\ApontamentoVeiculoGastoRepository;
use Schaffen\Gef\Services\ApontamentoVeiculoGastoService;
use App\Models\Financeiro\MovimentacaoFinanceira;
use App\Http\Controllers\Api\Financeiro\MvfController;
use Illuminate\Support\Facades\DB;
use App\Http\Utils\SCHException;
use Carbon\Carbon;
/**
 * Class ApontamentoVeiculoGastosController.
 *
 * @package namespace Schaffen\Gef\Http\Controllers;
 */
class ApontamentoVeiculoGastosController extends Controller
{
   /**
    * @var ApontamentoVeiculoGastoRepository
    */
   protected $repository;

   /**
    * @var ApontamentoVeiculoGastoService
    */
   protected $service;

   /**
    * ApontamentoVeiculoGastosController constructor.
    *
    * @param ApontamentoVeiculoGastoRepository $repository
    * @param ApontamentoVeiculoGastoService $service
    */
   public function __construct(ApontamentoVeiculoGastoRepository $repository, ApontamentoVeiculoGastoService $service)
   {
      $this->repository = $repository;
      $this->service = $service;
   }

   public function getConsumiveis()
   {
      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\ApontamentosComConsumiveisCriteria());
      $apontamento_veiculo_gasto = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código", 'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "descricao", 'title' => "Descricao", 'dataType' => 'text', 'filter' => ['descricao' => "text"], 'sortable' => "descricao", 'show' => true]);
      array_push($ret['cols'], ['field' => "responsavel", 'title' => "Responsavel", 'dataType' => 'text', 'filter' => ['responsavel' => "text"], 'sortable' => "responsavel", 'show' => true]);
      array_push($ret['cols'], ['field' => "veiculo", 'title' => "Veiculo", 'dataType' => 'text', 'filter' => ['veiculo' => "text"], 'sortable' => "veiculo", 'show' => true]);
      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);

      $ret['rows'] = $apontamento_veiculo_gasto["data"];

      return response()->json([
         'data' => $ret,
      ]);
   }
   public function getAcessorios()
   {

      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\ApontamentosComAcessoriosCriteria());
      $apontamento_veiculo_gasto = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código", 'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "descricao", 'title' => "Descricao", 'dataType' => 'text', 'filter' => ['descricao' => "text"], 'sortable' => "descricao", 'show' => true]);
      array_push($ret['cols'], ['field' => "responsavel", 'title' => "Responsavel", 'dataType' => 'text', 'filter' => ['responsavel' => "text"], 'sortable' => "responsavel", 'show' => true]);
      array_push($ret['cols'], ['field' => "veiculo", 'title' => "Veiculo", 'dataType' => 'text', 'filter' => ['veiculo' => "text"], 'sortable' => "veiculo", 'show' => true]);
      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);

      $ret['rows'] = $apontamento_veiculo_gasto["data"];

      return response()->json([
         'data' => $ret,
      ]);
   }

   /**
    * Display a listing of the resource.
    *
    * @return \Illuminate\Http\Response
    */
   public function index(Request $request)
   {
      $this->repository->pushCriteria(new \Schaffen\Gef\Criterias\RequestPrefixCriteria($request, ApontamentoVeiculoGasto::$prefix));
      $apontamento_veiculo_gasto = $this->repository->all();

      $ret['cols'] = array();
      array_push($ret['cols'], ['field' => "codigo", 'title' => "Código", 'dataType' => 'id', 'filter' => ['codigo' => "text"], 'sortable' => "codigo", 'show' => true]);
      array_push($ret['cols'], ['field' => "descricao", 'title' => "Descricao", 'dataType' => 'text', 'filter' => ['descricao' => "text"], 'sortable' => "descricao", 'show' => true]);
      array_push($ret['cols'], ['field' => "responsavel", 'title' => "Responsavel", 'dataType' => 'text', 'filter' => ['responsavel' => "text"], 'sortable' => "responsavel", 'show' => true]);
      array_push($ret['cols'], ['field' => "veiculo", 'title' => "Veiculo", 'dataType' => 'text', 'filter' => ['veiculo' => "text"], 'sortable' => "veiculo", 'show' => true]);
      array_push($ret['cols'], ['field' => "action", 'title' => "Opções", 'dataType' => 'command', 'show' => true]);
      $ret['rows'] = $apontamento_veiculo_gasto["data"];

      return response()->json([
         'data' => $ret,
      ]);

   }

   /**
    * Store a newly created resource in storage.
    *
    * @param  ApontamentoVeiculoGastoCreateRequest $request
    *
    * @return \Illuminate\Http\Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function store(ApontamentoVeiculoGastoCreateRequest $request)
   {
      $response = $this->service->updateOrCreate($request->all());
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }

   /**
    * Display the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */

   public function show($id)
   {
      $this->repository->setPresenter("Schaffen\\Gef\\Presenters\\ApontamentoVeiculoGastoShowPresenter");
      $apontamento_veiculo_gasto = $this->repository->find($id);

      return Response::response(true, null, $apontamento_veiculo_gasto, null);
   }

   /**
    * Show the form for editing the specified resource.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function edit($id)
   {
      $apontamentoVeiculoGasto = $this->repository->find($id);

      return view('apontamentoVeiculoGastos.edit', compact('apontamentoVeiculoGasto'));
   }

   /**
    * Update the specified resource in storage.
    *
    * @param  ApontamentoVeiculoGastoUpdateRequest $request
    * @param  string $id
    *
    * @return Response
    *
    * @throws \Prettus\Service\Exceptions\ServiceException
    */
   public function update(ApontamentoVeiculoGastoUpdateRequest $request, $id)
   {
      $response = $this->service->updateOrCreate($request->all(), $id);
      return Response::response($response['status'], $response['message'], $response['data'], $response['errors'], $response['http_code']);
   }


   /**
    * Remove the specified resource from storage.
    *
    * @param  int $id
    *
    * @return \Illuminate\Http\Response
    */
   public function destroy($id)
   {
      try{
         $consumivel = ApontamentoVeiculoGasto::find($id);
         $mvf = ApontamentoVeiculoGastoMvf::where('avgm_fk_avg_codigo',$id)->get();

         $consumivel->deleted_at = Carbon::now();

         if(!empty($mvf)){
            foreach($mvf as $m){
               $movfin = MovimentacaoFinanceira::find($m->avgm_fk_mvf_codigo);
               if(!empty($movfin)){
                  if($movfin->mvf_status != 2){
                     $mvfEstorno = new MvfController();
                     $m->avgm_fk_mvf_codigo = null;
                     $m->save();
                     $mvfEstorno->destroy($movfin->mvf_codigo);
                     $m->delete();

                  }else{
                     throw new SCHException("Não é possível remover o apontamento. A MVF código ".$movfin->mvf_codigo. " já encontra-se quitada.");
                  }
               }

            }
         }
         $consumivel->save();
         DB::commit();
         return Response::success('Apontamento removido com sucesso.','');
      }  catch (SCHException $e) {
           DB::rollback();
           return Response::faillure($e->getMessage(), $e->getTrace());
       } catch (\Exception $e) {
           DB::rollback();
           return Response::error($e->getMessage(), $e);
       }
   }

   public function buscarConsumiveis($consumivelCodigo){
      $consumivel = ApontamentoVeiculoGasto::select(DB::raw('(avgi_quantidade * avgi_valor_unitario) as valor,
                                                              responsavel.pes_nome as responsavel_nome,
                                                              responsavel.pes_codigo as responsavel_codigo,
                                                              fornecedor.pes_nome as fornecedor_nome,
                                                              fornecedor.pes_codigo as fornecedor_codigo,
                                                              avg_cobrar_cliente,
                                                              vei_fk_ccu_codigo,
                                                              vei_placa,
                                                              pro_descricao,
                                                              pro_fk_pcg_codigo_deb,
                                                              pro_fk_pcg_codigo_cred'))
                                          ->join('cadastro.pessoa as responsavel','responsavel.pes_codigo','=','avg_fk_responsavel_pes_codigo')
                                          ->join('cadastro.pessoa as fornecedor','fornecedor.pes_codigo','=','avg_fk_fornecedor_pes_codigo')
                                          ->join('frota.apontamento_veiculo_gasto_item','apontamento_veiculo_gasto_item.avgi_fk_avg_codigo','=','avg_codigo')
                                          ->join('frota.veiculo','veiculo.vei_codigo','=','avg_fk_vei_codigo')
                                          ->join('frota.produto_frota','produto_frota.prfr_codigo','=','avgi_fk_prfr_codigo')
                                          ->join('produto.produto','produto.pro_codigo','=','prfr_fk_pro_codigo')
                                          ->where('avg_codigo',$consumivelCodigo)
                                          ->get();

      return Response::response(true, null, $consumivel, null);
   }

   public function visualizarApontamento($codigo){

      $apontamento = ApontamentoVeiculoGasto::where('avg_codigo',$codigo)->with("responsavel","items","veiculo","fornecedor","items.produtofrota","items.produtofrota.produto","contrato","mvf")->get();
      return Response::response(true, null, $apontamento, null);

   }
}
